import {
  defineEventHandler,
  createError,
  getQuery,
  getMethod,
  readBody
} from "h3";
import {
  queryParamToVariables,
  getEndpoint,
  getFetchOptions,
  validateRequest
} from "./helpers";
import { getModuleConfig } from "./helpers/getModuleConfig.mjs";
import { GraphqlMiddlewareOperation } from "./../settings/index.mjs";
import { documents } from "#graphql-documents";
export default defineEventHandler(async (event) => {
  const method = getMethod(event);
  const operation = event.context.params.operation;
  const name = event.context.params.name;
  validateRequest(method, operation, name, documents);
  const query = documents[operation][name];
  const config = await getModuleConfig();
  const endpoint = getEndpoint(config, event, operation, name);
  const fetchOptions = getFetchOptions(config, event, operation, name);
  const variables = operation === GraphqlMiddlewareOperation.Query ? queryParamToVariables(getQuery(event)) : await readBody(event);
  return $fetch.raw(endpoint, {
    method: "POST",
    body: {
      query,
      variables
    },
    ...fetchOptions
  }).then((response) => {
    if (config.onServerResponse) {
      return config.onServerResponse(event, response, operation, name);
    }
    return response._data;
  }).catch((err) => {
    if (config.onServerError) {
      return config.onServerError(event, err, operation, name);
    }
    throw createError({
      statusCode: 500,
      statusMessage: "Couldn't execute GraphQL query.",
      data: err && "message" in err ? err.message : err
    });
  });
});
